/*
 * Generic pager functionality.
 *
 * Copyright (C) 2021, 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <systypes/base.h>

#include <fsserver/generic_page_mapper.h>
#include <resource/resource.h>



/* A pager exposing a dataspace. */

class Pager : public Resource
{
protected:
    offset_t _start, _size;
    GenericPageMapper *_mapper, *_mapper_base, *_mapper_masked, *_mapper_copied;
    map_flags_t _map_flags;

public:
    explicit Pager(GenericPageMapper *mapper, map_flags_t map_flags);

    virtual void close();

    /* Paging methods. */

    virtual long map(offset_t offset, map_address_t hot_spot, map_flags_t map_flags,
                     l4_snd_fpage_t *region);

    /* Limit methods. */

    offset_t get_data_size();

    /* File methods. */

    virtual long flush(offset_t position, offset_t *size);

    virtual long resize(offset_t *size);

    /* Mapped file methods. */

    virtual long mmap(offset_t position, offset_t length,
                      offset_t start_visible, offset_t end_visible,
                      offset_t *start_pos, offset_t *end_pos, offset_t *size);
};

// vim: tabstop=4 expandtab shiftwidth=4
